package com.ruoyi.common.utils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.time.zone.ZoneRules;
import java.util.Date;
import java.util.Locale;
import java.util.Optional;

import org.apache.commons.lang3.time.DateFormatUtils;

/**
 * 时间工具类
 * 
 * @author ruoyi
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM",
            "yyyy年MM月dd日"};

    /**
     * 获取当前Date型日期
     * 
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     * 
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }

    /**
     * 计算时间差
     *
     * @param endDate 最后时间
     * @param startTime 开始时间
     * @return 时间差（天/小时/分钟）
     */
    public static String timeDistance(Date endDate, Date startTime)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 增加 LocalDateTime ==> Date
     */
    public static Date toDate(LocalDateTime temporalAccessor)
    {
        ZonedDateTime zdt = temporalAccessor.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 增加 LocalDate ==> Date
     */
    public static Date toDate(LocalDate temporalAccessor)
    {
        LocalDateTime localDateTime = LocalDateTime.of(temporalAccessor, LocalTime.of(0, 0, 0));
        ZonedDateTime zdt = localDateTime.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    public static String CNTimeStr(Object now,String zone) {
        String nowTime = "";
        Date nowDate = new Date();
        //判断是否为日期
        if(now instanceof Date){
            nowDate = (Date) now;
        }else if(now instanceof String){
           if( Optional.ofNullable(parseDate(now)).isPresent()){
               nowDate = parseDate(now);
           }
        }
        nowTime = parseDateToStr(YYYY_MM_DD_HH_MM_SS,nowDate);
        ZoneId australia = ZoneId.of(zone);

        ZoneId SH = ZoneId.of("Asia/Shanghai");
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS);
        LocalDateTime localDate = LocalDateTime.parse(nowTime,dateTimeFormatter);

//        ZoneRules zoneRules = australia.getRules();
//        boolean isDaylightSavings = zoneRules.isDaylightSavings(localDate.toInstant(ZoneOffset.UTC));
//        Duration daylightSavingsOffset = Duration.ZERO;
//        if (isDaylightSavings) {
//            //计算夏令时的偏移量
//            daylightSavingsOffset = zoneRules.getDaylightSavings(localDate.toInstant(ZoneOffset.UTC));
//            //将偏移量应用到当前时间
//            localDate = localDate.plus(daylightSavingsOffset);
//        }
        ZonedDateTime zonedDateTime = ZonedDateTime.of(localDate, australia);
        //获取UTC时间
//        ZonedDateTime utcDate = zonedDateTime.withZoneSameInstant(ZoneOffset.UTC);
//        DateTimeFormatter UTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
//                .withZone(ZoneOffset.UTC);
        ZonedDateTime utcDate = zonedDateTime.withZoneSameInstant(SH);
        DateTimeFormatter UTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(SH);

//        ZonedDateTime dateAndTimeInSydney = ZonedDateTime.of(localDate, australia);
//        ZonedDateTime utcDate = dateAndTimeInSydney.withZoneSameInstant(ZoneOffset.UTC);


//        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
//                .withZone(australia);
//        ZonedDateTime localDate = ZonedDateTime.parse(nowTime,dateTimeFormatter);
//        //获取UTC时间
//        ZonedDateTime utcDate = localDate.withZoneSameInstant(ZoneOffset.UTC);
//        DateTimeFormatter UTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
//                .withZone(ZoneOffset.UTC);
        return utcDate.format(UTCFormat);
    }

    public static Date CNTimeDate(Object now,String zone) {
        String nowTime = "";
        Date nowDate = new Date();
        //判断是否为日期
        if(now instanceof Date){
            nowDate = (Date) now;
        }else if(now instanceof String){
            if( Optional.ofNullable(parseDate(now)).isPresent()){
                nowDate = parseDate(now);
            }
        }
        nowTime = parseDateToStr(YYYY_MM_DD_HH_MM_SS,nowDate);
        ZoneId australia = ZoneId.of(zone);

        ZoneId SH = ZoneId.of("Asia/Shanghai");

        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS);
        LocalDateTime localDate = LocalDateTime.parse(nowTime,dateTimeFormatter);

        ZonedDateTime zonedDateTime = ZonedDateTime.of(localDate, australia);
        //获取UTC时间
//        ZonedDateTime utcDate = zonedDateTime.withZoneSameInstant(ZoneOffset.UTC);
//        DateTimeFormatter UTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
//                .withZone(ZoneOffset.UTC);
        ZonedDateTime utcDate = zonedDateTime.withZoneSameInstant(SH);
        DateTimeFormatter UTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(SH);

        return parseDate(utcDate.format(UTCFormat));
    }
    public static Date CNTimeDate(Object now) {
        return CNTimeDate(now,SecurityUtils.getTimeZone());
    }

    public static String LocalTime(Object now,String zone) {
        String nowTime = "";
        Date nowDate = new Date();
        //判断是否为日期
        if(now instanceof Date){
            nowDate = (Date) now;
        }else if(now instanceof String){
            if( Optional.ofNullable(parseDate(now)).isPresent()){
                nowDate = parseDate(now);
            }
        }
        nowTime = parseDateToStr(YYYY_MM_DD_HH_MM_SS,nowDate);
        ZoneId australia = ZoneId.of(zone);
        ZoneId SH = ZoneId.of("Asia/Shanghai");
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(SH);
        DateTimeFormatter LocalFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(australia);

        ZonedDateTime localDate = ZonedDateTime.parse(nowTime,LocalFormat);
//        ZonedDateTime dateAndTimeInSydney = ZonedDateTime.of(localDate, australia);
        //获取UTC时间
        ZonedDateTime shDate = localDate.withZoneSameInstant(SH);
        return localDate.format(dateTimeFormatter);
    }
    public static String LocalTime(Object now) {
        return LocalTime(now,SecurityUtils.getTimeZone());
    }
    public static Date LocalTimeDate(Object now) {
        String s = LocalTime(now, SecurityUtils.getTimeZone());
        try {
            return parseDate(s, YYYY_MM_DD_HH_MM_SS);
        } catch (ParseException e) {
            return null;
        }
    }
    
    

    public static Date UTCTimeDate() {
        Date nowDate = new Date();
        String nowTime = parseDateToStr(YYYY_MM_DD_HH_MM_SS, nowDate);
        ZoneId australia = ZoneId.of(SecurityUtils.getTimeZone());
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(australia);
        ZonedDateTime localDate = ZonedDateTime.parse(nowTime,dateTimeFormatter);
        //获取UTC时间
        ZonedDateTime utcDate = localDate.withZoneSameInstant(ZoneOffset.UTC);

        DateTimeFormatter uTCFormat = DateTimeFormatter.ofPattern(YYYY_MM_DD_HH_MM_SS)
                .withZone(ZoneOffset.UTC);
        return parseDate(utcDate.format(uTCFormat));
    }

}
